﻿--[[
Name: Rating Buster
Revision: $Revision: 2 $
Developed by: Whitetooth@Cenarius (hotdogee@巴哈姆特baha)
Description: Converts combat ratings in tooltips into normal percentages.
]]

-----------------
-- Addon Setup --
-----------------
-- Version Info
local MAJOR_VERSION = "0.8.2"
--local MINOR_VERSION = tonumber((string.gsub("$Revision: 2 $", "^.-(%d+).-$", "%1")))
-- Librarys
local Compost = AceLibrary("Compost-2.0")
local L = AceLibrary("AceLocale-2.2"):new("RatingBuster")
-- AceAddon Initialization
RatingBuster = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceConsole-2.0", "AceEvent-2.0", "AceDebug-2.0")
RatingBuster.title = "Rating Buster"
RatingBuster.version = MAJOR_VERSION
--RatingBuster.date = string.gsub("$Date: 2006-11-05 15:10:31 -0700 (Sun, 03 Sep 2006) $", "^.-(%d%d%d%d%-%d%d%-%d%d).-$", "%1")

-- Set Debugging
RatingBuster:SetDebugging(false)


---------------------
-- Saved Variables --
---------------------
-- Register DB
RatingBuster:RegisterDB("RatingBusterDB")
-- Default values
RatingBuster.DBDefaults = {
	showItemLevel = true,
	showItemID = false,
	useRequiredLevel = true,
	showCritFromAgi = true,
	showSpellCritFromInt = true,
	textColor = {r = 1.0, g = 0.996,  b = 0.545, hex = "|cfffffe8b"},
	enableTextColor = true,
}
local _, class = UnitClass("player")
if class == "PRIEST" or class == "MAGE" or class == "WARLOCK" then
	RatingBuster.DBDefaults.showCritFromAgi = false
end
if class == "WARRIOR" or class == "ROGUE" then
	RatingBuster.DBDefaults.showSpellCritFromInt = false
end

-- Register Defaults
RatingBuster:RegisterDefaults("profile", RatingBuster.DBDefaults)


---------------------------
-- Slash Command Options --
---------------------------
local consoleOptions = {
	type = 'group',
	args = {
		itemid = {
			type = 'toggle',
			name = L["Show Item ID"],
			desc = L["Show the Item ID in tooltips"],
			get = function() return RatingBuster.db.profile.showItemID end,
			set = function(v) RatingBuster.db.profile.showItemID = v end,
		},
		itemlevel = {
			type = 'toggle',
			name = L["Show Item Level"],
			desc = L["Show the Item Level in tooltips"],
			get = function() return RatingBuster.db.profile.showItemLevel end,
			set = function(v) RatingBuster.db.profile.showItemLevel = v end,
		},
		usereqlv = {
			type = 'toggle',
			name = L["Use Required Level"],
			desc = L["Calculate using the required level if you are below the required level"],
			get = function() return RatingBuster.db.profile.useRequiredLevel end,
			set = function(v) RatingBuster.db.profile.useRequiredLevel = v end,
		},
		level = {
			type = 'range',
			name = L["Level"],
			desc = L["Changes the Level used in calculations"],
			get = function() return RatingBuster.targetLevel end,
			set = function(v) RatingBuster.targetLevel = v end,
			min = 10,
			max = 73, -- set to level cap + 3
			step = 1,
		},
		color = {
			type = 'group',
			name = L["Change text color"],
			desc = L["Changes the color of added text"],
			args = {
				pick = {
					type = 'execute',
					name = L["Pick color"],
					desc = L["Pick a color"],
					func = function()
						CloseMenus()
						ColorPickerFrame.func = function()
							RatingBuster.db.profile.textColor.r, RatingBuster.db.profile.textColor.g, RatingBuster.db.profile.textColor.b = ColorPickerFrame:GetColorRGB();
							RatingBuster.db.profile.textColor.hex = "|cff"..string.format("%02x%02x%02x", RatingBuster.db.profile.textColor.r * 255, RatingBuster.db.profile.textColor.g * 255, RatingBuster.db.profile.textColor.b * 255)
						end
						ColorPickerFrame:SetColorRGB(u.r, RatingBuster.db.profile.textColor.g, RatingBuster.db.profile.textColor.b);
						ColorPickerFrame.previousValues = {r = RatingBuster.db.profile.textColor.r, g = RatingBuster.db.profile.textColor.g, b = RatingBuster.db.profile.textColor.b};
						ColorPickerFrame:Show()
					end,
				},
				enable = {
					type = 'toggle',
					name = L["Enable color"],
					desc = L["Enable colored text"],
					get = function() return RatingBuster.db.profile.enableTextColor end,
					set = function(v) RatingBuster.db.profile.enableTextColor = v end,
				},
			},
		},
		stat = {
			type = 'group',
			name = L["Stat"],
			desc = L["Changes the display of base stats"],
			args = {
				agi = {
					type = 'group',
					name = L["Agility"],
					desc = L["Changes the display of agility"],
					args = {
						crit = {
							type = 'toggle',
							name = L["Show crit"],
							desc = L["Show crit from agility"],
							get = function() return RatingBuster.db.profile.showCritFromAgi end,
							set = function(v) RatingBuster.db.profile.showCritFromAgi = v end,
						},
					},
				},
				int = {
					type = 'group',
					name = L["Intellect"],
					desc = L["Changes the display of intellect"],
					args = {
						spellcrit = {
							type = 'toggle',
							name = L["Show spell crit"],
							desc = L["Show spell crit from intellect"],
							get = function() return RatingBuster.db.profile.showSpellCritFromInt end,
							set = function(v) RatingBuster.db.profile.showSpellCritFromInt = v end,
						},
					},
				},
			},
		},
	},
}

RatingBuster:RegisterChatCommand({"/rb", "/ratingbuster"}, consoleOptions)


-----------
-- Tools --
-----------
-- copyTable from AceDB
local function copyTable(to, from)
	setmetatable(to, nil)
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, from)
	return to
end


----------------------------
-- GetEffectFromRating Function --
----------------------------
--[[ Rating ID
CR_WEAPON_SKILL = 1;
CR_DEFENSE_SKILL = 2;
CR_DODGE = 3;
CR_PARRY = 4;
CR_BLOCK = 5;
CR_HIT_MELEE = 6;
CR_HIT_RANGED = 7;
CR_HIT_SPELL = 8;
CR_CRIT_MELEE = 9;
CR_CRIT_RANGED = 10;
CR_CRIT_SPELL = 11;
CR_HIT_TAKEN_MELEE = 12;
CR_HIT_TAKEN_RANGED = 13;
CR_HIT_TAKEN_SPELL = 14;
CR_CRIT_TAKEN_MELEE = 15;
CR_CRIT_TAKEN_RANGED = 16;
CR_CRIT_TAKEN_SPELL = 17;
CR_HASTE_MELEE = 18;
CR_HASTE_RANGED = 19;
CR_HASTE_SPELL = 20;
CR_WEAPON_SKILL_MAINHAND = 21;
CR_WEAPON_SKILL_OFFHAND = 22;
CR_WEAPON_SKILL_RANGED = 23;
--]]
	
-- Level 60 base rating
local RatingBase = {
	[CR_WEAPON_SKILL] = 2.5,
	[CR_DEFENSE_SKILL] = 1.5,
	[CR_DODGE] = 12,
	[CR_PARRY] = 20,
	[CR_BLOCK] = 5,
	[CR_HIT_MELEE] = 10,
	[CR_HIT_RANGED] = 10,
	[CR_HIT_SPELL] = 8,
	[CR_CRIT_MELEE] = 14,
	[CR_CRIT_RANGED] = 14,
	[CR_CRIT_SPELL] = 14,
	[CR_HIT_TAKEN_MELEE] = 10,
	[CR_HIT_TAKEN_RANGED] = 10,
	[CR_HIT_TAKEN_SPELL] = 8,
	[CR_CRIT_TAKEN_MELEE] = 25,
	[CR_CRIT_TAKEN_RANGED] = 25,
	[CR_CRIT_TAKEN_SPELL] = 25,
	[CR_HASTE_MELEE] = 20 / 3,
	[CR_HASTE_RANGED] = 20 / 3,
	[CR_HASTE_SPELL] = 20 / 3,
	[CR_WEAPON_SKILL_MAINHAND] = 2.5,
	[CR_WEAPON_SKILL_OFFHAND] = 2.5,
	[CR_WEAPON_SKILL_RANGED] = 2.5,
}

-- Formula reverse engineered by Whitetooth@Cenarius (hotdogee@巴哈姆特baha)
-- returns percentages
function RatingBuster.GetEffectFromRating(rNumber, rType, level)
	-- check for invalid input
	if type(rNumber) ~= "number" or rType < 1 or rType > 23 then return 0 end
	-- defaults to player level if not given
	level = level or UnitLevel("player")
	if level >= 60 then
		return rNumber/RatingBase[rType]*((-3/82)*level+(131/41))
	elseif level >= 10 then
		return rNumber/RatingBase[rType]/((1/52)*level-(8/52))
	else
		return rNumber/RatingBase[rType]/((1/52)*10-(8/52))
	end
end


--------------------------------
-- Agi to Melee Crit Function --
--------------------------------
-- Numbers reverse engineered by Whitetooth@Cenarius (hotdogee@巴哈姆特baha)
local CritPerAgi = {
	 [1] = {0.2500, 0.2174, 0.2840, 0.4476, 0.0909, 0.1663, 0.0771, 0.1500, 0.2020, },
	 [2] = {0.2381, 0.2070, 0.2834, 0.4290, 0.0909, 0.1663, 0.0771, 0.1500, 0.2020, },
	 [3] = {0.2381, 0.2070, 0.2711, 0.4118, 0.0909, 0.1583, 0.0771, 0.1429, 0.1923, },
	 [4] = {0.2273, 0.1976, 0.2530, 0.3813, 0.0865, 0.1583, 0.0735, 0.1429, 0.1923, },
	 [5] = {0.2174, 0.1976, 0.2430, 0.3677, 0.0865, 0.1511, 0.0735, 0.1429, 0.1836, },
	 [6] = {0.2083, 0.1890, 0.2337, 0.3550, 0.0865, 0.1511, 0.0735, 0.1364, 0.1836, },
	 [7] = {0.2083, 0.1890, 0.2251, 0.3321, 0.0865, 0.1511, 0.0735, 0.1364, 0.1756, },
	 [8] = {0.2000, 0.1812, 0.2171, 0.3217, 0.0826, 0.1446, 0.0735, 0.1364, 0.1756, },
	 [9] = {0.1923, 0.1812, 0.2051, 0.3120, 0.0826, 0.1446, 0.0735, 0.1304, 0.1683, },
	[10] = {0.1923, 0.1739, 0.1984, 0.2941, 0.0826, 0.1385, 0.0701, 0.1304, 0.1553, },
	[11] = {0.1852, 0.1739, 0.1848, 0.2640, 0.0826, 0.1385, 0.0701, 0.1250, 0.1496, },
	[12] = {0.1786, 0.1672, 0.1670, 0.2394, 0.0790, 0.1330, 0.0701, 0.1250, 0.1496, },
	[13] = {0.1667, 0.1553, 0.1547, 0.2145, 0.0790, 0.1330, 0.0701, 0.1250, 0.1443, },
	[14] = {0.1613, 0.1553, 0.1441, 0.1980, 0.0790, 0.1279, 0.0701, 0.1200, 0.1443, },
	[15] = {0.1563, 0.1449, 0.1330, 0.1775, 0.0790, 0.1231, 0.0671, 0.1154, 0.1346, },
	[16] = {0.1515, 0.1449, 0.1267, 0.1660, 0.0757, 0.1188, 0.0671, 0.1111, 0.1346, },
	[17] = {0.1471, 0.1403, 0.1194, 0.1560, 0.0757, 0.1188, 0.0671, 0.1111, 0.1303, },
	[18] = {0.1389, 0.1318, 0.1117, 0.1450, 0.0757, 0.1147, 0.0671, 0.1111, 0.1262, },
	[19] = {0.1351, 0.1318, 0.1060, 0.1355, 0.0727, 0.1147, 0.0671, 0.1071, 0.1262, },
	[20] = {0.1282, 0.1242, 0.0998, 0.1271, 0.0727, 0.1073, 0.0643, 0.1034, 0.1122, },
	[21] = {0.1282, 0.1208, 0.0962, 0.1197, 0.0727, 0.1073, 0.0643, 0.1000, 0.1122, },
	[22] = {0.1250, 0.1208, 0.0910, 0.1144, 0.0727, 0.1039, 0.0643, 0.1000, 0.1092, },
	[23] = {0.1190, 0.1144, 0.0872, 0.1084, 0.0699, 0.1039, 0.0643, 0.0968, 0.1063, },
	[24] = {0.1163, 0.1115, 0.0829, 0.1040, 0.0699, 0.1008, 0.0617, 0.0968, 0.1063, },
	[25] = {0.1111, 0.1087, 0.0797, 0.0980, 0.0699, 0.0978, 0.0617, 0.0909, 0.1010, },
	[26] = {0.1087, 0.1060, 0.0767, 0.0936, 0.0673, 0.0950, 0.0617, 0.0909, 0.1010, },
	[27] = {0.1064, 0.1035, 0.0734, 0.0903, 0.0673, 0.0950, 0.0617, 0.0909, 0.0985, },
	[28] = {0.1020, 0.1011, 0.0709, 0.0865, 0.0673, 0.0924, 0.0617, 0.0882, 0.0962, },
	[29] = {0.1000, 0.0988, 0.0680, 0.0830, 0.0649, 0.0924, 0.0593, 0.0882, 0.0962, },
	[30] = {0.0962, 0.0945, 0.0654, 0.0792, 0.0649, 0.0875, 0.0593, 0.0833, 0.0878, },
	[31] = {0.0943, 0.0925, 0.0637, 0.0768, 0.0649, 0.0875, 0.0593, 0.0833, 0.0859, },
	[32] = {0.0926, 0.0925, 0.0614, 0.0741, 0.0627, 0.0853, 0.0593, 0.0811, 0.0859, },
	[33] = {0.0893, 0.0887, 0.0592, 0.0715, 0.0627, 0.0831, 0.0571, 0.0811, 0.0841, },
	[34] = {0.0877, 0.0870, 0.0575, 0.0691, 0.0627, 0.0831, 0.0571, 0.0789, 0.0824, },
	[35] = {0.0847, 0.0836, 0.0556, 0.0664, 0.0606, 0.0792, 0.0571, 0.0769, 0.0808, },
	[36] = {0.0833, 0.0820, 0.0541, 0.0643, 0.0606, 0.0773, 0.0551, 0.0750, 0.0792, },
	[37] = {0.0820, 0.0820, 0.0524, 0.0628, 0.0606, 0.0773, 0.0551, 0.0732, 0.0777, },
	[38] = {0.0794, 0.0791, 0.0508, 0.0609, 0.0586, 0.0756, 0.0551, 0.0732, 0.0777, },
	[39] = {0.0781, 0.0776, 0.0493, 0.0592, 0.0586, 0.0756, 0.0551, 0.0714, 0.0762, },
	[40] = {0.0758, 0.0750, 0.0481, 0.0572, 0.0586, 0.0723, 0.0532, 0.0698, 0.0709, },
	[41] = {0.0735, 0.0737, 0.0470, 0.0556, 0.0568, 0.0707, 0.0532, 0.0682, 0.0696, },
	[42] = {0.0725, 0.0737, 0.0457, 0.0542, 0.0568, 0.0707, 0.0532, 0.0682, 0.0696, },
	[43] = {0.0704, 0.0713, 0.0444, 0.0528, 0.0551, 0.0693, 0.0532, 0.0667, 0.0685, },
	[44] = {0.0694, 0.0701, 0.0433, 0.0512, 0.0551, 0.0679, 0.0514, 0.0667, 0.0673, },
	[45] = {0.0676, 0.0679, 0.0421, 0.0497, 0.0551, 0.0665, 0.0514, 0.0638, 0.0651, },
	[46] = {0.0667, 0.0669, 0.0413, 0.0486, 0.0534, 0.0652, 0.0514, 0.0625, 0.0641, },
	[47] = {0.0649, 0.0659, 0.0402, 0.0474, 0.0534, 0.0639, 0.0498, 0.0625, 0.0641, },
	[48] = {0.0633, 0.0639, 0.0391, 0.0464, 0.0519, 0.0627, 0.0498, 0.0612, 0.0631, },
	[49] = {0.0625, 0.0630, 0.0382, 0.0454, 0.0519, 0.0627, 0.0498, 0.0600, 0.0621, },
	[50] = {0.0610, 0.0612, 0.0373, 0.0440, 0.0519, 0.0605, 0.0482, 0.0588, 0.0585, },
	[51] = {0.0595, 0.0604, 0.0366, 0.0431, 0.0505, 0.0594, 0.0482, 0.0577, 0.0577, },
	[52] = {0.0588, 0.0596, 0.0358, 0.0422, 0.0505, 0.0583, 0.0482, 0.0577, 0.0569, },
	[53] = {0.0575, 0.0580, 0.0350, 0.0412, 0.0491, 0.0583, 0.0467, 0.0566, 0.0561, },
	[54] = {0.0562, 0.0572, 0.0341, 0.0404, 0.0491, 0.0573, 0.0467, 0.0556, 0.0561, },
	[55] = {0.0549, 0.0557, 0.0334, 0.0394, 0.0478, 0.0554, 0.0467, 0.0545, 0.0546, },
	[56] = {0.0543, 0.0550, 0.0328, 0.0386, 0.0478, 0.0545, 0.0454, 0.0536, 0.0539, },
	[57] = {0.0532, 0.0544, 0.0321, 0.0378, 0.0466, 0.0536, 0.0454, 0.0526, 0.0531, },
	[58] = {0.0521, 0.0530, 0.0314, 0.0370, 0.0466, 0.0536, 0.0454, 0.0517, 0.0525, },
	[59] = {0.0510, 0.0524, 0.0307, 0.0364, 0.0454, 0.0528, 0.0441, 0.0517, 0.0518, },
	[60] = {0.0500, 0.0512, 0.0301, 0.0355, 0.0454, 0.0512, 0.0441, 0.0500, 0.0493, },
	[61] = {0.0469, 0.0491, 0.0297, 0.0334, 0.0443, 0.0496, 0.0435, 0.0484, 0.0478, },
	[62] = {0.0442, 0.0483, 0.0290, 0.0322, 0.0444, 0.0486, 0.0432, 0.0481, 0.0472, },
	[63] = {0.0418, 0.0472, 0.0284, 0.0307, 0.0441, 0.0470, 0.0424, 0.0470, 0.0456, },
	[64] = {0.0397, 0.0456, 0.0279, 0.0296, 0.0433, 0.0456, 0.0423, 0.0455, 0.0447, },
	[65] = {0.0377, 0.0446, 0.0273, 0.0286, 0.0426, 0.0449, 0.0422, 0.0448, 0.0438, },
	[66] = {0.0360, 0.0437, 0.0270, 0.0276, 0.0419, 0.0437, 0.0411, 0.0435, 0.0430, },
	[67] = {0.0344, 0.0425, 0.0264, 0.0268, 0.0414, 0.0427, 0.0412, 0.0436, 0.0424, },
	[68] = {0.0329, 0.0416, 0.0259, 0.0262, 0.0412, 0.0417, 0.0408, 0.0424, 0.0412, },
	[69] = {0.0315, 0.0408, 0.0254, 0.0256, 0.0410, 0.0408, 0.0404, 0.0414, 0.0406, },
	[70] = {0.0303, 0.0400, 0.0250, 0.0250, 0.0400, 0.0400, 0.0400, 0.0405, 0.0400, },
	[71] = {0.0297, 0.0393, 0.0246, 0.0244, 0.0390, 0.0392, 0.0396, 0.0396, 0.0394, },
	[72] = {0.0292, 0.0385, 0.0242, 0.0238, 0.0381, 0.0384, 0.0393, 0.0387, 0.0388, },
	[73] = {0.0287, 0.0378, 0.0238, 0.0233, 0.0372, 0.0377, 0.0389, 0.0379, 0.0383, },
}

local ClassNameToID = {
	["WARRIOR"] = 1,
	["PALADIN"] = 2,
	["HUNTER"] = 3,
	["ROGUE"] = 4,
	["PRIEST"] = 5,
	["SHAMAN"] = 6,
	["MAGE"] = 7,
	["WARLOCK"] = 8,
	["DRUID"] = 9,
}

function RatingBuster.GetCritFromAgi(agi, class, level)
	-- check for invalid input
	if type(agi) ~= "number" then return end
	-- if level is invalid input, default to player level
	if type(level) ~= "number" or level < 1 or level > 73 then
		level = UnitLevel("player")
	end
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[string.upper(class)] ~= nil then
		class = ClassNameToID[string.upper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	-- Calculate
	return agi * CritPerAgi[level][class]
end


--------------------------------
-- Int to Spell Crit Function --
--------------------------------
-- Numbers reverse engineered by Whitetooth@Cenarius (hotdogee@巴哈姆特baha)
local SpellCritPerInt = {
	 [1] = {0.0000, 0.0832, 0.0699, 0.0000, 0.1710, 0.1333, 0.1637, 0.1500, 0.1431, },
	 [2] = {0.0000, 0.0793, 0.0666, 0.0000, 0.1636, 0.1272, 0.1574, 0.1435, 0.1369, },
	 [3] = {0.0000, 0.0793, 0.0666, 0.0000, 0.1568, 0.1217, 0.1516, 0.1375, 0.1312, },
	 [4] = {0.0000, 0.0757, 0.0635, 0.0000, 0.1505, 0.1217, 0.1411, 0.1320, 0.1259, },
	 [5] = {0.0000, 0.0757, 0.0635, 0.0000, 0.1394, 0.1166, 0.1364, 0.1269, 0.1211, },
	 [6] = {0.0000, 0.0724, 0.0608, 0.0000, 0.1344, 0.1120, 0.1320, 0.1222, 0.1166, },
	 [7] = {0.0000, 0.0694, 0.0608, 0.0000, 0.1297, 0.1077, 0.1279, 0.1179, 0.1124, },
	 [8] = {0.0000, 0.0694, 0.0583, 0.0000, 0.1254, 0.1037, 0.1240, 0.1138, 0.1124, },
	 [9] = {0.0000, 0.0666, 0.0583, 0.0000, 0.1214, 0.1000, 0.1169, 0.1100, 0.1086, },
	[10] = {0.0000, 0.0666, 0.0559, 0.0000, 0.1140, 0.1000, 0.1137, 0.1065, 0.0984, },
	[11] = {0.0000, 0.0640, 0.0559, 0.0000, 0.1045, 0.0933, 0.1049, 0.0971, 0.0926, },
	[12] = {0.0000, 0.0616, 0.0538, 0.0000, 0.0941, 0.0875, 0.0930, 0.0892, 0.0851, },
	[13] = {0.0000, 0.0594, 0.0499, 0.0000, 0.0875, 0.0800, 0.0871, 0.0825, 0.0807, },
	[14] = {0.0000, 0.0574, 0.0499, 0.0000, 0.0784, 0.0756, 0.0731, 0.0767, 0.0750, },
	[15] = {0.0000, 0.0537, 0.0466, 0.0000, 0.0724, 0.0700, 0.0671, 0.0717, 0.0684, },
	[16] = {0.0000, 0.0537, 0.0466, 0.0000, 0.0684, 0.0666, 0.0639, 0.0688, 0.0656, },
	[17] = {0.0000, 0.0520, 0.0451, 0.0000, 0.0627, 0.0636, 0.0602, 0.0635, 0.0617, },
	[18] = {0.0000, 0.0490, 0.0424, 0.0000, 0.0597, 0.0596, 0.0568, 0.0600, 0.0594, },
	[19] = {0.0000, 0.0490, 0.0424, 0.0000, 0.0562, 0.0571, 0.0538, 0.0569, 0.0562, },
	[20] = {0.0000, 0.0462, 0.0399, 0.0000, 0.0523, 0.0538, 0.0505, 0.0541, 0.0516, },
	[21] = {0.0000, 0.0450, 0.0388, 0.0000, 0.0502, 0.0518, 0.0487, 0.0516, 0.0500, },
	[22] = {0.0000, 0.0438, 0.0388, 0.0000, 0.0470, 0.0500, 0.0460, 0.0493, 0.0477, },
	[23] = {0.0000, 0.0427, 0.0368, 0.0000, 0.0453, 0.0474, 0.0445, 0.0471, 0.0463, },
	[24] = {0.0000, 0.0416, 0.0358, 0.0000, 0.0428, 0.0459, 0.0422, 0.0446, 0.0437, },
	[25] = {0.0000, 0.0396, 0.0350, 0.0000, 0.0409, 0.0437, 0.0405, 0.0429, 0.0420, },
	[26] = {0.0000, 0.0387, 0.0341, 0.0000, 0.0392, 0.0424, 0.0390, 0.0418, 0.0409, },
	[27] = {0.0000, 0.0387, 0.0333, 0.0000, 0.0376, 0.0412, 0.0372, 0.0398, 0.0394, },
	[28] = {0.0000, 0.0370, 0.0325, 0.0000, 0.0362, 0.0394, 0.0338, 0.0384, 0.0384, },
	[29] = {0.0000, 0.0362, 0.0318, 0.0000, 0.0348, 0.0383, 0.0325, 0.0367, 0.0366, },
	[30] = {0.0000, 0.0347, 0.0304, 0.0000, 0.0333, 0.0368, 0.0312, 0.0355, 0.0346, },
	[31] = {0.0000, 0.0340, 0.0297, 0.0000, 0.0322, 0.0354, 0.0305, 0.0347, 0.0339, },
	[32] = {0.0000, 0.0333, 0.0297, 0.0000, 0.0311, 0.0346, 0.0294, 0.0333, 0.0325, },
	[33] = {0.0000, 0.0326, 0.0285, 0.0000, 0.0301, 0.0333, 0.0286, 0.0324, 0.0318, },
	[34] = {0.0000, 0.0320, 0.0280, 0.0000, 0.0289, 0.0325, 0.0278, 0.0311, 0.0309, },
	[35] = {0.0000, 0.0308, 0.0269, 0.0000, 0.0281, 0.0314, 0.0269, 0.0303, 0.0297, },
	[36] = {0.0000, 0.0303, 0.0264, 0.0000, 0.0273, 0.0304, 0.0262, 0.0295, 0.0292, },
	[37] = {0.0000, 0.0297, 0.0264, 0.0000, 0.0263, 0.0298, 0.0254, 0.0284, 0.0284, },
	[38] = {0.0000, 0.0287, 0.0254, 0.0000, 0.0256, 0.0289, 0.0248, 0.0277, 0.0276, },
	[39] = {0.0000, 0.0282, 0.0250, 0.0000, 0.0249, 0.0283, 0.0241, 0.0268, 0.0269, },
	[40] = {0.0000, 0.0273, 0.0241, 0.0000, 0.0241, 0.0272, 0.0235, 0.0262, 0.0256, },
	[41] = {0.0000, 0.0268, 0.0237, 0.0000, 0.0235, 0.0267, 0.0230, 0.0256, 0.0252, },
	[42] = {0.0000, 0.0264, 0.0237, 0.0000, 0.0228, 0.0262, 0.0215, 0.0248, 0.0244, },
	[43] = {0.0000, 0.0256, 0.0229, 0.0000, 0.0223, 0.0254, 0.0211, 0.0243, 0.0240, },
	[44] = {0.0000, 0.0256, 0.0225, 0.0000, 0.0216, 0.0248, 0.0206, 0.0236, 0.0233, },
	[45] = {0.0000, 0.0248, 0.0218, 0.0000, 0.0210, 0.0241, 0.0201, 0.0229, 0.0228, },
	[46] = {0.0000, 0.0245, 0.0215, 0.0000, 0.0206, 0.0235, 0.0197, 0.0224, 0.0223, },
	[47] = {0.0000, 0.0238, 0.0212, 0.0000, 0.0200, 0.0231, 0.0192, 0.0220, 0.0219, },
	[48] = {0.0000, 0.0231, 0.0206, 0.0000, 0.0196, 0.0226, 0.0188, 0.0214, 0.0214, },
	[49] = {0.0000, 0.0228, 0.0203, 0.0000, 0.0191, 0.0220, 0.0184, 0.0209, 0.0209, },
	[50] = {0.0000, 0.0222, 0.0197, 0.0000, 0.0186, 0.0215, 0.0179, 0.0204, 0.0202, },
	[51] = {0.0000, 0.0219, 0.0194, 0.0000, 0.0183, 0.0210, 0.0176, 0.0200, 0.0198, },
	[52] = {0.0000, 0.0216, 0.0192, 0.0000, 0.0178, 0.0207, 0.0173, 0.0195, 0.0193, },
	[53] = {0.0000, 0.0211, 0.0186, 0.0000, 0.0175, 0.0201, 0.0170, 0.0191, 0.0191, },
	[54] = {0.0000, 0.0208, 0.0184, 0.0000, 0.0171, 0.0199, 0.0166, 0.0186, 0.0186, },
	[55] = {0.0000, 0.0203, 0.0179, 0.0000, 0.0166, 0.0193, 0.0162, 0.0182, 0.0182, },
	[56] = {0.0000, 0.0201, 0.0177, 0.0000, 0.0164, 0.0190, 0.0154, 0.0179, 0.0179, },
	[57] = {0.0000, 0.0198, 0.0175, 0.0000, 0.0160, 0.0187, 0.0151, 0.0176, 0.0176, },
	[58] = {0.0000, 0.0191, 0.0170, 0.0000, 0.0157, 0.0182, 0.0149, 0.0172, 0.0173, },
	[59] = {0.0000, 0.0189, 0.0168, 0.0000, 0.0154, 0.0179, 0.0146, 0.0168, 0.0169, },
	[60] = {0.0000, 0.0185, 0.0164, 0.0000, 0.0151, 0.0175, 0.0143, 0.0165, 0.0164, },
	[61] = {0.0000, 0.0157, 0.0157, 0.0000, 0.0148, 0.0164, 0.0143, 0.0159, 0.0162, },
	[62] = {0.0000, 0.0153, 0.0154, 0.0000, 0.0145, 0.0159, 0.0143, 0.0154, 0.0157, },
	[63] = {0.0000, 0.0148, 0.0150, 0.0000, 0.0143, 0.0152, 0.0143, 0.0148, 0.0150, },
	[64] = {0.0000, 0.0143, 0.0144, 0.0000, 0.0139, 0.0147, 0.0142, 0.0143, 0.0146, },
	[65] = {0.0000, 0.0140, 0.0141, 0.0000, 0.0137, 0.0142, 0.0142, 0.0138, 0.0142, },
	[66] = {0.0000, 0.0136, 0.0137, 0.0000, 0.0134, 0.0138, 0.0138, 0.0135, 0.0137, },
	[67] = {0.0000, 0.0133, 0.0133, 0.0000, 0.0132, 0.0134, 0.0133, 0.0130, 0.0133, },
	[68] = {0.0000, 0.0131, 0.0130, 0.0000, 0.0130, 0.0131, 0.0131, 0.0127, 0.0131, },
	[69] = {0.0000, 0.0128, 0.0128, 0.0000, 0.0127, 0.0128, 0.0128, 0.0125, 0.0128, },
	[70] = {0.0000, 0.0125, 0.0125, 0.0000, 0.0125, 0.0125, 0.0125, 0.0122, 0.0125, },
	[71] = {0.0000, 0.0122, 0.0123, 0.0000, 0.0123, 0.0122, 0.0122, 0.0119, 0.0122, },
	[72] = {0.0000, 0.0120, 0.0120, 0.0000, 0.0121, 0.0120, 0.0119, 0.0116, 0.0120, },
	[73] = {0.0000, 0.0118, 0.0118, 0.0000, 0.0119, 0.0117, 0.0117, 0.0114, 0.0118, },
}

function RatingBuster.GetSpellCritFromInt(int, class, level)
	-- check for invalid input
	if type(int) ~= "number" then return end
	-- if level is invalid input, default to player level
	if type(level) ~= "number" or level < 1 or level > 73 then
		level = UnitLevel("player")
	end
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[string.upper(class)] ~= nil then
		class = ClassNameToID[string.upper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	-- Calculate
	-- defaults to player level if not given
	return int * SpellCritPerInt[level][class]
end


---------------------
-- Initializations --
---------------------
function RatingBuster:OnInitialize()
	-- Hooking moved from OnEnable to OnInitialize until AceHook is fixed, to prevent multi hooking when resuming from standby
	-- Hook tooltip SetX methods
	self:HookTooltipMethods()
	-- OnUpdate Frame
	self.OnUpdateFrame = CreateFrame("Frame")
	self.OnUpdateFrame:SetScript("OnUpdate", self.OnUpdate)
end

function RatingBuster:OnEnable()
	-- Initialize targetLevel
	RatingBuster.targetLevel = UnitLevel("player")
	-- Check player level for lowbies
	if RatingBuster.targetLevel < 10 then
		RatingBuster.targetLevel = 10
	end
	-- for setting a new level
	self:RegisterEvent("PLAYER_LEVEL_UP")
end

function RatingBuster:OnDisable()
end

-- event = PLAYER_LEVEL_UP
-- arg1 = New player level
function RatingBuster:PLAYER_LEVEL_UP()
	RatingBuster.targetLevel = arg1
end


---------------
-- Schedules --
---------------
-- Schedule Table
RatingBuster.ScheduledTasks = {}
function RatingBuster:Schedule(taskName, timeAfter, functionName, ...)
	if (taskName and timeAfter) then -- functionName not required so we can use IsScheduled as a timer check
		Compost:Reclaim(self.ScheduledTasks[taskName], 2)
		self.ScheduledTasks[taskName] = Compost:AcquireHash(
			"TargetTime", GetTime() + timeAfter,
			"FunctionName", functionName,
			"Arg", Compost:Acquire(...)
		)
	end
end

function RatingBuster:ScheduleRepeat(taskName, repeatRate, functionName, ...)
	if (taskName and repeatRate and functionName) then -- functionName required
		Compost:Reclaim(self.ScheduledTasks[taskName], 2)
		self.ScheduledTasks[taskName] = Compost:AcquireHash(
			"Elapsed", 0,
			"RepeatRate", repeatRate,
			"FunctionName", functionName,
			"Arg", Compost:Acquire(...)
		)
	end
end

function RatingBuster:UnSchedule(taskName)
	--WT_RaidWarningAPI.Announce({HOTDOG = taskName})
	if not taskName then return end
	Compost:Reclaim(self.ScheduledTasks[taskName], 2)
	self.ScheduledTasks[taskName] = nil
end

function RatingBuster:IsScheduled(taskName)
	return (self.ScheduledTasks[taskName] ~= nil)
end

function RatingBuster:OnUpdate(elapsed)
	--RatingBuster:Debug("update: "..tostring(elapsed))
	local currentTime = GetTime()
	for taskName, task in pairs(RatingBuster.ScheduledTasks) do
		if type(task.TargetTime) ~= "nil" and (currentTime >= task.TargetTime) then
			RatingBuster:UnSchedule(taskName)
			if (task.FunctionName) then
				task.FunctionName(unpack(task.Arg))
			end
		elseif type(task.Elapsed) ~= "nil" then
			task.Elapsed = task.Elapsed + arg1
			if (task.Elapsed >= task.RepeatRate) then
				task.FunctionName(unpack(task.Arg))
				task.Elapsed = task.Elapsed - task.RepeatRate
			end
		end
	end
end


-------------------------------
-- Hook Tooltip SetX Methods --
-------------------------------
RatingBuster.TooltipList = {
	ItemRefTooltip,
	GameTooltip,
	ShoppingTooltip1,
	ShoppingTooltip2,
	--EquipCompare support
	ComparisonTooltip1,
	ComparisonTooltip2,
	--EQCompare support
	EQCompareTooltip1,
	EQCompareTooltip2,
	-- socketing interface
	--ItemSocketingDescription, -- commented out cause can't get it to work
	-- LinkWrangler support
	IRR_ItemRefTooltip1,
	IRR_ItemCompTooltip1,
	IRR_ItemCompTool11,
	IRR_ItemRefTooltip2,
	IRR_ItemCompTooltip2,
	IRR_ItemCompTool12,
	IRR_ItemRefTooltip3,
	IRR_ItemCompTooltip3,
	IRR_ItemCompTool13,
	IRR_ItemRefTooltip4,
	IRR_ItemCompTooltip4,
	IRR_ItemCompTool14,
	IRR_ItemRefTooltip5,
	IRR_ItemCompTooltip5,
	IRR_ItemCompTool15,
	-- MultiTips support
	ItemRefTooltip2,
	ItemRefTooltip3,
	ItemRefTooltip4,
	ItemRefTooltip5,
}

RatingBuster.MethodList = {
	"SetHyperlink",
	"SetBagItem",
	"SetInventoryItem",
	-- auction
	"SetAuctionItem",
	"SetAuctionSellItem",
	"SetAuctionCompareItem",
	-- loot
	"SetLootItem",
	"SetLootRollItem",
	-- crafting
	"SetCraftSpell",
	"SetCraftItem",
	"SetTradeSkillItem",
	"SetTrainerService",
	-- mail
	"SetInboxItem",
	"SetSendMailItem",
	-- buffs
	"SetPlayerBuff",
	-- quest log
	"SetQuestItem",
	"SetQuestLogItem",
	-- trade
	"SetTradePlayerItem",
	"SetTradeTargetItem",
	-- vendor tooltip
	"SetMerchantItem",
	"SetMerchantCompareItem",
	"SetBuybackItem",
	-- socketing interface
	--"SetSocketedItem", -- main display
	"SetSocketGem",
	"SetExistingSocketGem",
}

function RatingBuster.SetItem(tooltip)
	if not tooltip.GetItem then return end
	local _, link = tooltip:GetItem()
	RatingBuster.ProcessTooltip(tooltip, link)
end

function RatingBuster:HookTooltipMethods()
	for _, tooltip in pairs(self.TooltipList) do
		if tooltip then
			for _, methodName in ipairs(self.MethodList) do
				hooksecurefunc(tooltip, methodName, RatingBuster.SetItem)
				-- prints out an error when put on standby, as of Rev: 15813
				--self:SecureHook(tooltip, methodName, self.ProcessTooltip)
			end
		end
	end
end


--------------------------
-- Process Tooltip Core --
--------------------------
--[[
"+15 Agility"
-> "+15 Agility (+0.46% Crit)"
"+15 Crit Rating"
-> "+15 Crit Rating (+1.20%)"
"Equip: Increases your hit rating by 10."
-> "Equip: Increases your hit rating by 10 (+1.20%)."
--]]
local calcLevel, _
function RatingBuster.ProcessTooltip(tooltip, link)
	-- Check if we're in standby mode
	if not RatingBuster:IsActive() then return end
	-- Set calculation level
	if RatingBuster.db.profile.useRequiredLevel and link then
		_, _, _, _, calcLevel = GetItemInfo(link)
		--RatingBuster:Print(link..", "..calcLevel)
		if calcLevel and calcLevel <= RatingBuster.targetLevel then
			calcLevel = RatingBuster.targetLevel
		end
	end
	---------------------
	-- Tooltip Scanner --
	---------------------
	-- Loop through tooltip lines starting at line 2
	for i = 2, tooltip:NumLines() do
		local fontString = getglobal(tooltip:GetName().."TextLeft"..i)
		local text = fontString:GetText()
		if text and strfind(text, "%d") then -- do nothing if we don't find a number
			-- Initial pattern check, do nothing if not found
			-- Check for separators and bulid separatorTable
			local separatorTable = {}
			for _, sep in ipairs(L["separators"]) do
				if strfind(text, sep) then
					tinsert(separatorTable, sep)
				end
			end
			-- SplitDoJoin
			text = RatingBuster:SplitDoJoin(text, separatorTable)
			-- SetText
			fontString:SetText(text)
		end
	end
	----------------------------
	-- Item Level and Item ID --
	----------------------------
	-- Check for ItemLevel addon, do nothing if found
	if not ItemLevel_AddInfo and (RatingBuster.db.profile.showItemLevel or RatingBuster.db.profile.showItemID)then
		local id, level
		-- Get the Item ID from the link string
		if link then
			_, link, _, level = GetItemInfo(link)
			if link then
				_, _, id = strfind(link, "item:(%d+)")
			end
			local newLine = ""
			if level and RatingBuster.db.profile.showItemLevel then
				newLine = newLine..L["ItemLevel: "]..level
			end
			if id and RatingBuster.db.profile.showItemID then
				if newLine ~= "" then
					newLine = newLine..", "
				end
				newLine = newLine..L["ItemID: "]..id
			end
			if newLine ~= "" then
				tooltip:AddLine(newLine)
			end
		end
	end
	---------------------
	-- Repaint tooltip --
	---------------------
	tooltip:Show()
end

---------------------------------------------------------------------------------
-- Recursive algorithm that divides a string into pieces using the separators in separatorTable, 
-- processes them separately, then joins them back together
---------------------------------------------------------------------------------
-- text = "+24 Agility/+4 Stamina and +4 Spell Crit/+5 Spirit"
-- separatorTable = {"/", " and ", ","}
-- RatingBuster:SplitDoJoin("+24 Agility/+4 Stamina, +4 Dodge and +4 Spell Crit/+5 Spirit", {"/", " and ", ","})
--
function RatingBuster:SplitDoJoin(text, separatorTable)
	if type(separatorTable) == "table" and table.maxn(separatorTable) > 0 then
		local sep = tremove(separatorTable, 1)
		text =  string.gsub(text, sep, "@")
		text = {strsplit("@", text)}
		local processedText = {}
		local tempTable = {}
		for _, t in ipairs(text) do
			--self:Print(t[1])
			copyTable(tempTable, separatorTable)
			tinsert(processedText, self:SplitDoJoin(t, tempTable))
		end
		-- Join text
		return (string.gsub(strjoin("@", unpack(processedText)), "@", sep))
	else
		return self:ProcessText(text)
	end
end

function RatingBuster:ProcessText(text)
	-- 	Check if test has a matching pattern
	for _, num in ipairs(L["numberPatterns"]) do
		-- Convert text to lower so we don't have to worry about same ratings with different cases
		local lowerText = string.lower(text)
		-- Capture the stat value
		local s, e, value = strfind(lowerText, num.pattern)
		if value then
			-- Capture the stat name
			for _, stat in ipairs(L["statList"]) do
				if strfind(lowerText, stat.pattern) then
					local infoString = ""
					if type(stat.id) == "number" and stat.id >= 1 and stat.id <= 23 then
						--------------------
						-- Combat Ratings --
						--------------------
						-- Calculate stat value
						local effect = RatingBuster.GetEffectFromRating(tonumber(value), stat.id, calcLevel)
						--RatingBuster:Debug(reversedAmount..", "..amount..", "..v[2]..", "..RatingBuster.targetLevel)-- debug
						-- If rating is resilience, add a minus sign
						if stat.id >= 15 and stat.id <= 17 then
							effect = effect * -1
						end
						--RatingBuster.GetEffectFromRating(160, 18, 70)
						self:Debug(text..", "..tostring(effect)..", "..value..", "..stat.id..", "..calcLevel)
						-- Build info string
						infoString = format("%+.2f", effect)
						if stat.id > 2 and stat.id < 21 then -- if rating is a percentage
							infoString = infoString.."%"
						end
					elseif stat.id == SPELL_STAT2_NAME then
						-------------
						-- Agility --
						-------------
						local infoTable = {}
						if RatingBuster.db.profile.showCritFromAgi then
							local effect = RatingBuster.GetCritFromAgi(tonumber(value), class, calcLevel)
							if effect > 0 then
								tinsert(infoTable, (string.gsub(L["$value% Crit"], "$value", format("%+.2f", effect))))
							end
						end
						infoString = strjoin(", ", unpack(infoTable))
					elseif stat.id == SPELL_STAT4_NAME then
						---------------
						-- Intellect --
						---------------
						local infoTable = {}
						if RatingBuster.db.profile.showSpellCritFromInt then
							local effect = RatingBuster.GetSpellCritFromInt(tonumber(value), class, calcLevel)
							if effect > 0 then
								tinsert(infoTable, (string.gsub(L["$value% Spell Crit"], "$value", format("%+.2f", effect))))
							end
						end
						infoString = strjoin(", ", unpack(infoTable))
					end
					if infoString ~= "" then
						-- Add parenthesis
						infoString = "("..infoString..")"
						-- Add Color
						if RatingBuster.db.profile.enableTextColor then
							infoString = RatingBuster.db.profile.textColor.hex..infoString.."|r"
						end
						-- Build replacement string
						if num.addInfo == "AfterNumber" then -- Add after number
							infoString = string.gsub(infoString, "%%", "%%%%%%%%")
							infoString = string.gsub(strsub(text, s, e), "%d+", "%0 "..infoString)
						else -- Add after stat
							infoString = string.gsub(infoString, "%%", "%%%%")
							s, e = strfind(lowerText, stat.pattern)
							infoString = "%0 "..infoString
						end
						-- Insert info into text
						return (string.gsub(text, strsub(text, s, e), infoString, 1)) -- because gsub has 2 return values, but we only want 1
					end
					return text
				end
			end
		end
	end
	return text
end

-- RatingBuster:Bench(1000)
---------
-- self:SplitDoJoin("+24 Agility/+4 Stamina, +4 Dodge and +4 Spell Crit/+5 Spirit", {"/", " and ", ","})
-- 1000 times: 0.16 - 0.18 without Compost
-- 1000 times: 0.22 - 0.24 with Compost
---------
-- RatingBuster.ProcessTooltip(ItemRefTooltip, link)
-- 1000 times: 0.31 sec - 0.7.6
-- 1000 times: 0.29 sec - 0.8.0
---------
function RatingBuster:Bench(k)
	local t1 = GetTime()
	local link = GetInventoryItemLink("player", 12)
	for i = 1, k, 1 do
		---------------------------------------------------------------------------
		--self:SplitDoJoin("+24 Agility/+4 Stamina, +4 Dodge and +4 Spell Crit/+5 Spirit", {"/", " and ", ","})
		---------------------------------------------------------------------------
		ItemRefTooltip:SetInventoryItem("player", 12)
		RatingBuster.ProcessTooltip(ItemRefTooltip, link)
		---------------------------------------------------------------------------
		---------------------------------------------------------------------------
	end
	return GetTime() - t1
end
